/*
 * function to compute a great circle arc on a sphere
 * 
 * after the fortran routine by Dave Harris to do the same.
 * 
 */

#include <math.h>
#include "invsubs.h"

#define pi 3.1415926
#define pi_over_2  1.570796327
#define pi_over_4  0.785398163
#define deg_to_rad 0.017453292
#define rad_to_deg 57.29577951


/*
 * given two locations on a sphere, compute the great circle arc
 * 
 * dist is in km, az is in degrees
 * 
 * input and output are in degrees
 * 
 * lat1,lon1 are the starting points
 * 
 * lat2,lon2 are the last points
 * 
 * npts is the number of points used to construct the arc
 * 
 * lats, lons - output arrays containing the arc
 * 
 */

void Gcarc(float lat1, float lon1, float lat2, float lon2, float az, float dist, float *lats, float *lons, int npts)
{

	float           dl, theta, ct, st, phi, cp, sp, bearing;
	float           cb, sb, d, cd, sd, delta, earth_radius;
	float           ez, ex, ey;
	int              i;

	earth_radius = 6378.2064;
	dl = dist / (npts - 1);

	theta = deg_to_rad * lat1;
	ct = cos(theta);
	st = sin(theta);

	phi = deg_to_rad * lon1;
	cp = cos(phi);
	sp = sin(phi);

	bearing = deg_to_rad * az;
	cb = cos(bearing);
	sb = sin(bearing);

	lats[0] = lat1;
	lons[0] = lon1;
	d = 0;
	for (i = 1; i < npts - 1; i++) {
		d += dl;
		delta = (d / earth_radius);
		cd = cos(delta);
		sd = sin(delta);
		ez = cd * st + sd * cb * ct;
		ey = cd * ct * cp + sd * (-cb * st * cp - sb * sp);
		ex = cd * ct * sp + sd * (-cb * st * sp + sb * cp);

		lats[i] = (float) (rad_to_deg * atan2(ez, sqrt(ex * ex + ey * ey)));
		lons[i] = (float) (rad_to_deg * atan2(ex, ey));
	}
	lats[npts - 1] = lat2;
	lons[npts - 1] = lon2;

}
