/**********************************************************************************************/

/* function to get path lengths from ray paths  */

/**********************************************************************************************/

#include <math.h>
#include <stdio.h>
#include "invsubs.h"



void BuildFatRay(float *xp, float *zp, int nsegs, int nx, int nz, float x_inc, float z_inc, float wavelength, float arcDistance, float *lengths)
{
    int             i, j, k;
    float           z0,x0,xr,zr,theDist,dl,sum,w2;


    w2 = wavelength*wavelength;
    sum = 0;
    for (i = 0; i < nx * nz; i++)
        lengths[i] = 0;

    /* calculate the distance from the center of each cell in the image to the ray path */
    for(i = 0; i< nx; i++)
    {
        x0 = (i + 0.5) * x_inc;
        for(j = 0; j < nz; j++)
        {
            z0 = (j + 0.5) * z_inc;
            theDist = 1e9;
            for(k = 0; k < nsegs; k+=2)
            {
                xr = xp[k];
                zr = zp[k];
                
                dl = sqrt((x0-xr)*(x0-xr)+(z0-zr)*(z0-zr));
                
                if(dl < theDist)
                    theDist = dl;
            }
            /* compute a "fat" ray with a two dimensional Gaussian function */
            if(dl <= wavelength)
            {
                /* value is e^-5 when distance is wavelength/3 */
                lengths[j*nx+i] = exp(-15.0 * (dl*dl)/w2);
                sum += lengths[j*nx+i];
            }
        }
    }
    /* insure that the sum of the fat ray components is equal to the arc distance */
    /* this makes the method predict the correct time in a uniform model */
    if(sum > 0.0)
    {
        theDist = arcDistance / sum;
        for(i=0;i<nx*nz;i++)
            lengths[i] *= theDist;
    }

}

/**********************************************************************************************/

/* function to get path lengths from ray paths  */

/**********************************************************************************************/

void GetLengths(float *xp, float *zp, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths)
{
	int             i, lastcell, cell;
	float           arc_length = 0, dx, dz;
	int             ixb, izb;


	for (i = 0; i < nx * nz; i++)
		lengths[i] = 0;

	/* find which cell you are in */
	ixb = (int) (xp[0] / x_inc);
	izb = (int) (zp[0] / z_inc);
	lastcell = izb * nx + ixb;
	arc_length = 0;

	for (i = 1; i < nsegs; i++) {
		dz = (zp[i] - zp[i - 1]);
		dx = (xp[i] - xp[i - 1]);
		arc_length += sqrt(dz * dz + dx * dx);
		cell = ((int) (zp[i] / z_inc)) * nx + ((int) (xp[i] / x_inc));

		if (cell != lastcell) {
			lastcell = cell;
			lengths[lastcell] = arc_length;
			arc_length = 0;
		}
	}
	/* finish up in the last cell */
	if (arc_length != 0)
		lengths[cell] = arc_length;

}

/**********************************************************************************************/

/* function to get great-circle arc lengths in each cell  */

/**********************************************************************************************/

void GetGCARCLengths(float *xp, float *zp, float *lats, float *lons, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths)
{
	int             i, lastcell, cell;
	float           arc_length = 0, dx, dz, az,total_length;
	int             ixb, izb;
	int             i_enter;


	for (i = 0; i < nx * nz; i++)
		lengths[i] = 0;
	total_length = 0;

	/* find which cell you are in */
	ixb = (int) (xp[0] / x_inc);
	izb = (int) (zp[0] / z_inc);
	lastcell = izb * nx + ixb;
	arc_length = 0;
	i_enter = 0;
	

	for (i = 1; i < nsegs; i++) {
		dz = (zp[i] - zp[i - 1]);
		dx = (xp[i] - xp[i - 1]);
		cell = ((int) (zp[i] / z_inc)) * nx + ((int) (xp[i] / x_inc));

		if (cell != lastcell) {
			delta_az(&lats[i_enter], &lons[i_enter], &lats[i], &lons[i], &arc_length, &az);
			lengths[lastcell] = arc_length * 111.19;
			i_enter = i;
			total_length += arc_length * 111.19;
			lastcell = cell;
		}
	}
	/* finish up in the last cell */
	if (i_enter != nsegs - 1) {
		delta_az(&lats[i_enter], &lons[i_enter], &lats[nsegs - 1], &lons[nsegs - 1], &arc_length, &az);
		lengths[cell] = arc_length * 111.19;
		total_length += arc_length * 111.19;
	}
}
