#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <stdlib.h>
#include "invsubs.h"

/*
 * Function to smooth an image using the indices from the Laplacian smoother
 * routine.  the smoothed image overwrites the original image.
 * 
 */

void SmoothImage(float *inbuf, float vwt, float hwt, int nr, int nc, int npass)
{
	int            *theIndices, nnot_zero, i, j, ipass;
	float          *theFloats, *outbuf, avg, smoother = 1;


	if ((nr == 1 && nc == 1) || (npass == 0) )
		return;
	

	theFloats = (float *) malloc(5 * sizeof(float));
	theIndices = (int *) malloc(5 * sizeof(int));
	outbuf = (float *) malloc(nr * nc * sizeof(float));
	theFloats -= 1;
	theIndices -= 1;
	outbuf -= 1;


	/* just using the indices from the Get2DsmootherValues */

	for (ipass = 1; ipass <= npass; ipass++) {

		for (i = 1; i <= nr * nc; i++) {

			Get2DsmootherValues(smoother, vwt, hwt, i, nr, nc, theFloats, theIndices, &nnot_zero);

			/* adjust so that indexing begins with zero */

			avg = 0.0;

			for (j = 1; j <= nnot_zero; j++)
				avg += inbuf[theIndices[j] - 1];

			/* store the average value */

			outbuf[i] = avg / (float) nnot_zero;
		}

		for (i = 0; i < nr * nc; i++)
			inbuf[i] = outbuf[i + 1];

	}

	theFloats += 1;
	theIndices += 1;
	outbuf += 1;
	free((char *) theFloats);
	free((char *) theIndices);
	free((char *) outbuf);

	return;
}


/*
 * Function to smooth an image using the indices from the Laplacian smoother
 * routine.  the smoothed image overwrites the original image.
 * 
 */

void DistanceSmoothImage(float *inbuf, float maxdist, int nr, int nc, int npass)
{
	int            *theIndices, nnot_zero, i, j, ipass;
	float          *theFloats, *outbuf, avg, smoother = 1;


	if ((nr == 1 && nc == 1) || (npass == 0) )
		return;
	

	theFloats = (float *) malloc(nr*nc * sizeof(float));
	theIndices = (int *) malloc(nr*nc * sizeof(int));
	outbuf = (float *) malloc(nr * nc * sizeof(float));
	theFloats -= 1;
	theIndices -= 1;
	outbuf -= 1;


	/* just using the indices from the Get2DsmootherValues */

	for (ipass = 1; ipass <= npass; ipass++) {

		for (i = 1; i <= nr * nc; i++) {

			GetDsmootherValues(smoother, maxdist, i, nr, nc, theFloats, theIndices, &nnot_zero);

			/* adjust so that indexing begins with zero */

			avg = 0.0;

			for (j = 1; j <= nnot_zero; j++)
				avg += inbuf[theIndices[j] - 1];

			/* store the average value */

			outbuf[i] = avg / (float) nnot_zero;
		}

		for (i = 0; i < nr * nc; i++)
			inbuf[i] = outbuf[i + 1];

	}

	theFloats += 1;
	theIndices += 1;
	outbuf += 1;
	free((char *) theFloats);
	free((char *) theIndices);
	free((char *) outbuf);

	return;
}


/*
 * Function to smooth an image using the indices from the Laplacian smoother
 * routine.  the smoothed image overwrites the original image.
 * 
 * Uses a crude median smoothness criteria - inflexible, always 3by3
 * 
 */

void MedianSmoothImage(float *inbuf, float vwt, float hwt, int nr, int nc, int npass)
{
	int            *theIndices, nnot_zero, i, j, ipass;
	float          *theFloats, *outbuf, avg, smoother = 1, median;
	void            mdian1();


	if ((nr == 1 && nc == 1) || (npass == 0))
		return;

	theFloats = (float *) malloc(5 * sizeof(float));
	theIndices = (int *) malloc(5 * sizeof(int));
	outbuf = (float *) malloc(nr * nc * sizeof(float));
	theFloats -= 1;
	theIndices -= 1;
	outbuf -= 1;


	/* just using the indices from the Get2DsmootherValues */

	for (ipass = 1; ipass <= npass; ipass++) {

		for (i = 1; i <= nr * nc; i++) {

			Get2DsmootherValues(smoother, vwt, hwt, i, nr, nc, theFloats, theIndices, &nnot_zero);

			/* store the values in the tmp array theFloats */
			for (j = 1; j <= nnot_zero; j++)
				theFloats[j] = inbuf[theIndices[j] - 1];

			/*
			 * call the numerical recipes routine to get the
			 * median
			 */

			mdian1(theFloats, nnot_zero, &median);

			/* store the average value */

			outbuf[i] = median;
		}

		for (i = 0; i < nr * nc; i++)
			inbuf[i] = outbuf[i + 1];

	}

	theFloats += 1;
	theIndices += 1;
	outbuf += 1;
	free((char *) theFloats);
	free((char *) theIndices);
	free((char *) outbuf);

	return;
}
/*
 * Function to smooth an image using the indices from the Laplacian smoother
 * routine.  the smoothed image overwrites the original image.
 * 
 * Uses a crude median smoothness criteria - inflexible, always 3by3
 * 
 */

void DistanceMedianSmoothImage(float *inbuf, float maxdist, int nr, int nc, int npass)
{
	int            *theIndices, nnot_zero, i, j, ipass;
	float          *theFloats, *outbuf, avg, smoother = 1, median;
	void            mdian1();


	if ((nr == 1 && nc == 1) || (npass == 0))
		return;

	theFloats = (float *) malloc(nr*nc * sizeof(float));
	theIndices = (int *) malloc(nr*nc * sizeof(int));
	outbuf = (float *) malloc(nr * nc * sizeof(float));
	theFloats -= 1;
	theIndices -= 1;
	outbuf -= 1;


	/* just using the indices from the Get2DsmootherValues */

	for (ipass = 1; ipass <= npass; ipass++) {

		for (i = 1; i <= nr * nc; i++) {

			GetDsmootherValues(smoother, maxdist, i, nr, nc, theFloats, theIndices, &nnot_zero);

			/* store the values in the tmp array theFloats */
			for (j = 1; j <= nnot_zero; j++)
				theFloats[j] = inbuf[theIndices[j] - 1];

			/*
			 * call the numerical recipes routine to get the
			 * median
			 */

			mdian1(theFloats, nnot_zero, &median);

			/* store the average value */

			outbuf[i] = median;
		}

		for (i = 0; i < nr * nc; i++)
			inbuf[i] = outbuf[i + 1];

	}

	theFloats += 1;
	theIndices += 1;
	outbuf += 1;
	free((char *) theFloats);
	free((char *) theIndices);
	free((char *) outbuf);

	return;
}


/* FROM NUMERICAL RECIPES */

void mdian1(float *x, int n, float *xmed)
{
	int             n2, n2p;
	void            sort();

	sort(n, x);
	n2p = (n2 = n / 2) + 1;
	*xmed = (n % 2 ? x[n2p] : 0.5 * (x[n2] + x[n2p]));
}
void sort(int n, float *ra)
{
	int             l, j, ir, i;
	float           rra;

	l = (n >> 1) + 1;
	ir = n;
	for (;;) {
		if (l > 1)
			rra = ra[--l];
		else {
			rra = ra[ir];
			ra[ir] = ra[1];
			if (--ir == 1) {
				ra[1] = rra;
				return;
			}
		}
		i = l;
		j = l << 1;
		while (j <= ir) {
			if (j < ir && ra[j] < ra[j + 1])
				++j;
			if (rra < ra[j]) {
				ra[i] = ra[j];
				j += (i = j);
			} else
				j = ir + 1;
		}
		ra[i] = rra;
	}
}
