#include <stdio.h>

struct complex {
        float re;
        float im;
};

/****************************************************************************/
/* define a structure to hold the observations */
/****************************************************************************/

struct OBSERVATION {
  float evlat;
  float evlon;
  float stlat;
  float stlon;
  float phs_velocity;
  float phs_velocity_uncertainty;
  float weight;
  float period;
  int   iperiod;
  char  event_id[32];
  int   eventID;
  int   stationID;
  char  wavetype;
};

float GetMisfit(float *s,int nCells, int nobs,float *obs);

void GetPerturbations(float *perturbation,int nCells,int nImages,int nr,int nc,float alpha,float maxpert, long *seed);
void ZeroIt(float *x,int n);
void SumThem(float *x,float *y,int n);
void CopyIt(float *x,float *y,int n);
float GetMisfit(float *s,int nCells, int nobs,float *obs);

void LocalSearch(float *s,int nobs,float *obs,int nlooks,float alpha, float maxpert,int nCells,int nImages,int nr,int nc,long *seed);
void cfft2d(struct complex *x,int nx);
void cfft(struct complex *x,int n,int isign);
void dbpt(double lat, double lon, double a[4]);
void delta_az(float *alati, float *aloni, float *blati, float *bloni, float *delta, float *baz);
void Gcarc(float lat1, float lon1, float lat2, float lon2, float az, float dist, float *lats, float *lons, int npts);

void Get2Dsmoother(float smoother, float vwt, float hwt, int whichParameter, int nr, int nc, float *theRow);
void Get2DsmootherValues(float smoother, float vwt, float hwt, int whichParameter, int nr, int nc, float *theRow, int *indices, int *nnot_zero);
void GetDsmootherValues(float smoother, float maxdist, int whichParameter, int nr, int nc, float *theRow, int *indices, int *nnot_zero);
void Append2Dsmoother(FILE *intfile, FILE *flofile, float vwt, float hwt, int nr, int nc, float smoother);
void MultiAppend2Dsmoother(FILE *intfile, FILE *flofile, float vwt, float hwt, int nr, int nc, float smoother,int nImages);
void GetLengths(float *xp, float *zp, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths);
void BuildFatRay(float *xp, float *zp, int nsegs, int nx, int nz, float x_inc, float z_inc, float wavelength, float arcDistance, float *lengths);
void GetGCARCLengths(float *xp, float *zp, float *lats, float *lons, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths);
void GetPerturbation(float *y, int nh, int nd, float alpha, float amp,long *seed);
void getrand(float *x,int n, long *seed);
void AddPerturbation(float *buffer1,float *buffer2, int Npts);
void CopyImage(float *buffer1,float *buffer2, int nelements);
float GetRandomNumber(long *idum);
void GetModelRough(int m,float *s,int nx,int ny,float *meanvel, float *roughvel);
void GetARough(float *pS, int nx, int ny, float *meanvel, float *roughvel);
void SmoothImage(float *inbuf, float vwt, float hwt, int nr, int nc, int npass);
void DistanceSmoothImage(float *inbuf, float maxdist, int nr, int nc, int npass);
void MedianSmoothImage(float *inbuf, float vwt, float hwt, int nr, int nc, int npass);
void DistanceMedianSmoothImage(float *inbuf, float maxdist, int nr, int nc, int npass);
void mdian1(float *x, int n, float *xmed);
void sort(int n, float *ra);
int inRectangle(float x, float y, float l, float r, float t, float b );
void LATLON_to_XY(float lat, float lon, float *x, float *y, float minlat, float center_lon, float min_x);
void DiskLSQR(char *fn, int nr, int nc, float *x, float *b, int max_iterations, float damper, float max_cond_num, float *residual);
void DiskAVPU(char *fn, int nr, int nc, float *u, float *v);
void DiskATUPV(char *fn, int nr, int nc, float *u, float *v);
void WDiskLSQR(char *fn, int nr, int nc, float *x, float *b, float *wts, int max_iterations, float damper, float max_cond_num, float *residual);
void WDiskAVPU(char *fn, float *wts, int nr, int nc, float *u, float *v);
void WDiskATUPV(char *fn, float *wts, int nr, int nc, float *u, float *v);
void nrerror(char *error_text);
float *vector(int nl, int nh);
int *ivector(int nl, int nh);
double *dvector(int nl, int nh);
float **matrix(int nrl, int nrh, int ncl, int nch, int *m_error);
double **dmatrix(int nrl, int nrh, int ncl, int nch);
int **imatrix(int nrl, int nrh, int ncl, int nch);
float **submatrix(float **a, int oldrl, int oldrh, int oldcl, int oldch, int newrl, int newcl);
void free_vector(float *v, int nl, int nh);
void free_ivector(int *v, int nl, int nh);
void free_dvector(double *v, int nl, int nh);
void free_matrix(float **m, int nrl, int nrh, int ncl, int nch);
void free_dmatrix(double **m, int nrl, int nrh, int ncl, int nch);
void free_imatrix(int **m, int nrl, int nrh, int ncl, int nch);
void free_submatrix(float **b, int nrl, int nrh, int ncl, int nch);
float **convert_matrix(float *a, int nrl, int nrh, int ncl, int nch);
void free_convert_matrix(float **b, int nrl, int nrh, int ncl, int nch);
void Normalize(float *vector, int dim, float *length);
struct OBSERVATION nReadAnObservation(FILE *fd0);
struct OBSERVATION ReadAnObservation(FILE *fd0);
float ran2(long *idum);
void PutrowP(FILE *intfile, FILE *flofile, float *theRow, int nc, float effective_zero);
void Putrow0(FILE *intfile, FILE *flofile, float *theRow, int nc, float effective_zero);
void PutRowValues(FILE *intfile, FILE *flofile, float *theFloats, int *theIndices, int *n_notZero);
void Getrow(FILE *intfile, FILE *flofile, float *theRow, int nc, int *ierror);
void GetRowValues(FILE *intfile, FILE *flofile, float *theFloats, int *theIndices, int *nnzero, int *ierror);
void WriteModels(int iteration, float *coverage, float *s, float *v, float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x, int nModels);
void WriteAModel(int mnumber,int iteration,float *coverage, float *s, float *v, float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x);
void WriteModelGrid(float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x);
void WtDiskLSQR(char *fn, int nr, int nc, float *x, float *b, int max_iterations, float damper, float max_cond_num, float *residual,float *wt);
void WtDiskAVPU(char *fn, int nr, int nc, float *u, float *v, float *wt);
void WtDiskATUPV(char *fn, int nr, int nc, float *u, float *v, float *wt);
