#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <sys/types.h>
#include <sys/uio.h>
#include "invsubs.h"


main(ac, av)
	int             ac;
	char          **av;
{
	float           a[100], x[10], b[10], estimate[10], predicted[10], wts[10],
	                res, ran2();
	float           lsqr_res, damper, max_condition, *p1, *p2, *p3, b_copy[10];
	float epsilon = 0.10;
	int             nr = 10, nc = 5, i, j, iteration;
	long            idum = 1961;
	FILE           *intfile, *flofile;

	damper = 0.0001;

	for (i = 0; i < nr; i++)
		wts[i] = 1;

	for (i = 0; i < nr * nc; i++) {
		a[i] = ran2(&idum);
	}

	for (i = 0; i < nc; i++)
		x[i] = ran2(&idum);

	for (j = 0; j < nr; j++) {
		b[j] = 0;
		for (i = 0; i < nc; i++) {
			b[j] += a[j * nc + i] * x[i];
		}
		b_copy[j] = b[j];
	}

	/* make an outlier */
	b[4] *= 50;
	b_copy[4] = b[4];

	intfile = fopen("L_indices", "w");
	if (intfile == NULL) {
		fprintf(stderr, "Problem opening the path index file.\n");
		exit(-1);
	}
	flofile = fopen("L_floats", "w");
	if (flofile == NULL) {
		fprintf(stderr, "Problem opening the path float file.\n");
		exit(-1);
	}
	for (j = 0; j < nr; j++)
		Putrow0(intfile, flofile, &a[j * nc], nc, (float) 0.00001);

	fclose(intfile);
	fclose(flofile);

	p1 = &b[0];
	p1 -= 1;
	p2 = &estimate[0];
	p2 -= 1;
	p3 = &wts[0];
	p3 -= 1;

	fprintf(stdout, "LS Inversion\n\n");

	DiskLSQR(NULL, nr, nc, p2, p1, (int) 15, damper, (float) 2000, &lsqr_res);


	for (j = 0; j < nr; j++) {
		b[j] = b_copy[j];
		predicted[j] = 0;
		for (i = 0; i < nc; i++) {
			predicted[j] += a[j * nc + i] * estimate[i];
		}

	}


	fprintf(stdout, "True x,  Estimated x, Difference\n");
	for (i = 0; i < nc; i++)
		fprintf(stdout, "%f %f %10.5e\n", x[i], estimate[i], x[i] - estimate[i]);

	fprintf(stdout, "\nObservation,  Prediction, Difference\n");
	for (i = 0; i < nr; i++)
		fprintf(stdout, "%f %f %10.5e\n", b_copy[i], predicted[i], b_copy[i] - predicted[i]);


	fprintf(stdout, "\n\nIteratively Reweighted LS Inversion\n\n");
	/* iterative least-squares */
	for (iteration = 0; iteration < 15; iteration++) {

		WtDiskLSQR(NULL, nr, nc, p2, p1, (int) 15, damper, (float) 2000, &lsqr_res, p3);

		for (j = 0; j < nr; j++) {
			b[j] = b_copy[j];
			predicted[j] = 0;
			for (i = 0; i < nc; i++) {
				predicted[j] += a[j * nc + i] * estimate[i];
			}

			res = ( fabs(predicted[j] - b[j]) );
			fprintf(stderr,"res %d = %f\n",j,res);
			if (res > epsilon)
				wts[j] = epsilon / res;
			else
				wts[j] = 1;

		}

	}

	fprintf(stdout, "True x,  Estimated x, Difference\n");
	for (i = 0; i < nc; i++)
		fprintf(stdout, "%f %f %10.5e\n", x[i], estimate[i], x[i] - estimate[i]);


	fprintf(stdout, "\nObservation,  Prediction, Difference\n");
	for (i = 0; i < nr; i++)
		fprintf(stdout, "%f %f %10.5e %f\n", b_copy[i], predicted[i], b_copy[i] - predicted[i], wts[i]);

}


#include <math.h>
#include <stdio.h>

#define M 714025
#define IA 1366
#define IC 150889

float
ran2(idum)
	long           *idum;
{
	static long     iy, ir[98];
	static int      iff = 0;
	int             j;
	void            nrerror();

	if (*idum < 0 || iff == 0) {
		iff = 1;
		if ((*idum = (IC - (*idum)) % M) < 0)
			*idum = -(*idum);
		for (j = 1; j <= 97; j++) {
			*idum = (IA * (*idum) + IC) % M;
			ir[j] = (*idum);
		}
		*idum = (IA * (*idum) + IC) % M;
		iy = (*idum);
	}
	j = 1 + 97.0 * iy / M;
	if (j > 97 || j < 1)
		fprintf(stdout, "RAN2: This cannot happen.");
	iy = ir[j];
	*idum = (IA * (*idum) + IC) % M;
	ir[j] = (*idum);
	return (float) iy / M;
}

#undef M
#undef IA
#undef IC
